using System;

using Microsoft.Xna.Framework.Input;

using Syntriax.Engine.Core;
using Syntriax.Engine.Input;

namespace Pong.Behaviours;

public class PongScoreboard(int WinScore) : BehaviourOverride
{
    public Action? OnReset { get; set; } = null;
    public Action? OnFinished { get; set; } = null;
    public Action? OnScored { get; set; } = null;

    public int ScoreLeft { get; private set; } = 0;
    public int ScoreRight { get; private set; } = 0;
    public int ScoreSum => ScoreLeft + ScoreRight;

    public int WinScore { get; } = WinScore;


    protected override void OnFirstActiveFrame()
    {
        KeyboardInputsBehaviour? buttonInputs = null!;

        if (!BehaviourController.TryGetBehaviour(out buttonInputs))
            buttonInputs = BehaviourController.AddBehaviour<KeyboardInputsBehaviour>();

        buttonInputs.RegisterOnRelease(Keys.Space, (_, _1) => Reset());
    }


    public void ScoreToLeft()
    {
        ScoreLeft++;
        OnScored?.Invoke();

        CheckFinish();
    }

    public void ScoreToRight()
    {
        ScoreRight++;
        OnScored?.Invoke();

        CheckFinish();
    }

    public void Reset()
    {
        ScoreLeft = ScoreRight = 0;
        OnReset?.Invoke();
    }

    private void CheckFinish()
    {
        int halfwayScore = (int)(WinScore * .5f);

        if (ScoreLeft > halfwayScore || ScoreRight > halfwayScore)
            OnFinished?.Invoke();
    }
}
